#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 27 15:26:30 2017

    Uma implementação de front-end gráfico em QML e pyqt5 para o Game Of Life
    do prof. Newton Maruyama.
    Esta implementação busca deliberadamente minimizar a quantidade de código
    em JavaScript (para evitar confundir os alunos com duas linguagens).
    Provavelmente seria mais eficiente fazer o engine qml processar o conteúdo
    da matriz do jogo em javascript diretamente.

@author: thiago
"""
import sys
from PyQt5.QtWidgets import QApplication
from PyQt5.QtQml import QQmlApplicationEngine, qmlRegisterType
from PyQt5.QtCore import QObject, Qt, QVariant, QAbstractListModel, QModelIndex, pyqtProperty, pyqtSignal, pyqtSlot, QTimer

import JogoDaVidaAluno

class Cell(QObject):
    
    def __init__(self,  i, j, matrix, parent=None):
        super().__init__(parent)
        self._i = i
        self._j = j
        self._matrix = matrix
    
    aliveChanged = pyqtSignal()
    @pyqtProperty(bool,notify=aliveChanged)
    def alive(self):
        return self._matrix[self._j][self._i]

  
class GameModel(QAbstractListModel):
    aliveRole = Qt.UserRole + 1
    def __init__(self, parent=None, *args): 
        super(GameModel, self).__init__(parent)
        self._rows = 30
        self._cols = 30
        self._matrix =[[False for i in range(self._cols)] for j in range(self._rows)]
        self._Cells = [[Cell(i, j, self._matrix) for i in range(self._cols)] for j in range(self._rows)]
        self._timer = QTimer()
        self._timer.timeout.connect(self.gameStep)
        self._speed = 120
        self._timer.setInterval(60000//self._speed)
        self._playing = False
    
    @pyqtSlot(int)
    def toogleCell(self, index):
        i = index %  self._cols
        j = index // self._rows
        self._matrix[j][i] ^= True
        self._Cells[j][i].aliveChanged.emit()
    
    @pyqtSlot()
    def gameStep(self):
        newMatrix = JogoDaVidaAluno.UpdateMatrix(self._matrix)
        for j in range(len(self._matrix)):
            for i in range(len(self._matrix[j])):
                self._matrix[j][i] = newMatrix[j][i]
        self.dataChanged.emit(self.createIndex(0,0),self.createIndex(self._rows*self._cols,0))
                
    @pyqtSlot()
    def gameGo(self):
        self._playing = True
        self._timer.start()
        self.playingChanged.emit()
        
    @pyqtSlot()
    def gameStop(self):
        self._playing = False
        self._timer.stop()
        self.playingChanged.emit()
        
    playingChanged = pyqtSignal()
    @pyqtProperty(bool,notify=playingChanged)
    def playing(self):
        return self._playing  
    
    speedChanged = pyqtSignal()
    @pyqtProperty(int,notify=speedChanged)
    def speed(self):
        return self._speed
    @speed.setter
    def setSpeed(self,value):
        self._speed = value
        self._timer.setInterval(60000//self._speed) 
        
    rowsChanged = pyqtSignal()
    @pyqtProperty(int,notify=rowsChanged)
    def rows(self):
        return self._rows
    
    colsChanged = pyqtSignal()    
    @pyqtProperty(int, notify=colsChanged)
    def cols(self):
        return self._cols
    
    def rowCount(self, parent=QModelIndex()):
        return self._rows*self._cols
        
    def data(self, index, role=Qt.DisplayRole):
        if role == self.aliveRole:
            i = index.row() %  self._cols
            j = index.row() // self._rows
            return self._Cells[j][i]
        else:
            return QVariant()
        
    def roleNames(self):
        names = {}
        names[self.aliveRole] = b"alive"
        return names
    
        
# Main Function
if __name__ == '__main__':
    # Create main app
    myApp = QApplication(sys.argv)
    
    qmlRegisterType(Cell, 'Cell', 1, 0, 'Cell')
    
    # Load QML
    engine = QQmlApplicationEngine()
    game = GameModel()
    engine.rootContext().setContextProperty("game", game)
    
    engine.load('qml/GoLGrid.qml')
    # Create Window
    window = engine.rootObjects()[0]    
    #print(QMetaObject.invokeMethod(array,'itemAt(int)', Qt.BlockingQueuedConnection, Q_RETURN_ARG(int), Q_ARG(int, 0)))
    # Execute the Application and Exit
    sys.exit(myApp.exec_())
